/**********************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.smlif.internal.editor;


import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.cosmos.rm.smlif.internal.SMLImages;
import org.eclipse.cosmos.rm.smlif.internal.SMLPlugin;
import org.eclipse.cosmos.rm.smlif.internal.common.SMLCommonUtil;
import org.eclipse.cosmos.rm.smlif.internal.common.SMLMessages;
import org.eclipse.cosmos.rm.smlif.internal.dialogs.ResourceSelectionDialog;
import org.eclipse.cosmos.rm.validation.internal.common.ISMLConstants;
import org.eclipse.cosmos.rm.validation.internal.common.IValidationConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ColumnPixelData;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableLayout;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.MenuAdapter;
import org.eclipse.swt.events.MenuEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class DocumentsPage extends AbstractFormPage implements SelectionListener, ISelectionChangedListener
{
	/**
	 * The id of this page
	 */
	public static final String ID = SMLPlugin.PLUGIN_ID + ".DocumentsPage"; //$NON-NLS-1$
	
	/**
	 * Operations
	 */
	private static final byte OPERATION_ADD_DOCUMENT = 0x00;
	private static final byte OPERATION_REMOVE_DOCUMENT = 0x01;
	private static final byte OPERATION_ADD_ALIASES = 0x02;
	private static final byte OPERATION_REMOVE_ALIASES = 0x03;
	private static final byte OPERATION_MODIFY_ALIAS = 0x04;
	
	/**
	 * The aliases table
	 */
	private Table aliasTable;
	
	/**
	 * The style text representing the raw representation of the document
	 */
	private StyledText rawStyledText;

	/**
	 * The logical representation of a document
	 */
	private TreeViewer logicalRepresentation;

	/**
	 * The node representing the current document selection
	 */
	private Node input;

	/**
	 * The add alias button
	 */
	private Button addAlias;

	/**
	 * The remove alias button
	 */
	private Button removeAlias;


	/**
	 * Keeps track of the indices for an element selected in
	 * the logical tree representation of a document.
	 * KEY = An object of type {@link String} indicating the index of an element
	 * VALUE = int[2], where int[0] indicates the starting index and int[1]
	 * indicates the end index.
	 */
	private Map indices;
	
	/**
	 * The logical tree viewer
	 */
	private TreeViewer documentTreeViewer;
	
	
	/**
	 * Constructor
	 * 
	 * @param editor The editor containing this form page
	 */
	public DocumentsPage(FormEditor editor)
	{
		super(editor, ID, SMLMessages.editorDocumentTitle);
		indices = new Hashtable();
	}
	
	protected void setWeight(SashForm sashForm)
	{
		sashForm.setWeights(new int[]{40, 60});
	}
	
	protected void createLeftColumn(Composite parent)
	{
 		getManagedForm().addPart(((SMLIFEditor)getEditor()).getSmlifFormPart());
		FormToolkit toolkit = getManagedForm().getToolkit();
		Section section = createSection(parent);
		
		Composite sectionClient = toolkit.createComposite(section, SWT.NONE);
		sectionClient.setLayout(new GridLayout());
		sectionClient.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		
		/* The tree */
		Tree documentTree = toolkit.createTree(sectionClient, SWT.NONE);
		documentTree.setLayout(new GridLayout());
		documentTree.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		documentTreeViewer = new TreeViewer(documentTree);		
		documentTreeViewer.setContentProvider(new DocumentContentProvider());
		documentTreeViewer.setLabelProvider(new DocumentLabelProvider());		
		documentTreeViewer.addSelectionChangedListener(this);
		documentTreeViewer.setInput(getEditor());
		
		/* The context menu of the document tree */
		Menu documentContextMenu = new Menu(documentTree);
		documentTree.setMenu(documentContextMenu);
		
		final MenuItem addDocument = createMenuItem(documentContextMenu, SMLMessages.editorAddDocument, OPERATION_ADD_DOCUMENT);	
		final MenuItem removeDocument = createMenuItem(documentContextMenu, SMLMessages.editorRemoveDocument, OPERATION_REMOVE_DOCUMENT);	
	
		documentContextMenu.addMenuListener(new MenuAdapter()
		{

			public void menuShown(MenuEvent e)
			{
				DocumentTreeElement selection = (DocumentTreeElement)((StructuredSelection)documentTreeViewer.getSelection()).getFirstElement();
				boolean rootSelected = selection != null && selection.isRootElement();
				
				addDocument.setEnabled(rootSelected);
				addDocument.setImage(rootSelected ? SMLImages.INSTANCE.getImage("c", SMLImages.ADD) : SMLImages.INSTANCE.getImage("d", SMLImages.ADD)); //$NON-NLS-1$ //$NON-NLS-2$
					
				removeDocument.setEnabled(!rootSelected);
				removeDocument.setImage(!rootSelected ? SMLImages.INSTANCE.getImage("c", SMLImages.REMOVE) : SMLImages.INSTANCE.getImage("d", SMLImages.REMOVE)); //$NON-NLS-1$ //$NON-NLS-2$
			}
		});
		
		makeSectionReady (section, sectionClient, SMLMessages.editorDocuments);
	}

	private MenuItem createMenuItem(Menu parent, String text, byte operation)
	{
		final MenuItem menuItem = new MenuItem(parent, SWT.NONE);
		menuItem.setText(text);
		menuItem.addSelectionListener(this);
		menuItem.setData(new Byte(operation));
		
		return menuItem;
	}

	private Button createButton(Composite parent, String text, SelectionListener listener, byte operation, boolean enabled)
	{
		Button button = getManagedForm().getToolkit().createButton(parent, text, SWT.NONE);
		button.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		button.addSelectionListener(listener);
		button.setData(new Byte(operation));
		button.setEnabled(enabled);
		
		GridData layoutData = new GridData(SWT.DEFAULT, SWT.DEFAULT, true, false);
		layoutData.widthHint = 100;
		layoutData.horizontalAlignment = SWT.RIGHT;
		button.setLayoutData(layoutData);
		
		return button;
	}


	/**
	 * @see org.eclipse.cosmos.rm.smlif.internal.editor.AbstractFormPage#createRightColumn(org.eclipse.swt.widgets.Composite)
	 */
	protected void createRightColumn(Composite parent)
	{
		SMLFormToolkit toolkit = ((SMLIFEditor)getEditor()).getExtendedToolkit();		
		SashForm sashForm = toolkit.createSashForm(parent, SWT.VERTICAL);
		sashForm.setLayout(new GridLayout());
		sashForm.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		Section logicalSection = createSection(sashForm);
		
		Composite logicalClient = toolkit.createComposite(logicalSection);
		logicalClient.setLayout(new GridLayout());
		logicalClient.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		
		toolkit.createLabel(logicalClient, SMLMessages.editorAliases);
		
		Composite aliasComposite = toolkit.createComposite(logicalClient);
		aliasComposite.setLayout(new GridLayout(2, false));
		aliasComposite.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		
		aliasTable = toolkit.createTable(aliasComposite, SWT.NONE);
		aliasTable.setLinesVisible(true);
		aliasTable.setLayout(new GridLayout());
		aliasTable.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));
		aliasTable.setHeaderVisible(true);				
		TableViewer aliasTableViewer = new CheckboxTableViewer(aliasTable);
		setTableProperties(aliasTableViewer);
		
		Composite buttonComposite = toolkit.createComposite(aliasComposite);
		buttonComposite.setLayout(new GridLayout());
		GridData buttonCompositeGD = new GridData();
		buttonCompositeGD.verticalAlignment = SWT.TOP;
		buttonCompositeGD.horizontalAlignment = SWT.RIGHT;
		buttonComposite.setLayoutData(buttonCompositeGD);
		addAlias = createButton(buttonComposite, SMLMessages.editorAddButton, this, OPERATION_ADD_ALIASES, false);
		removeAlias = createButton(buttonComposite, SMLMessages.editorRemoveButton, this, OPERATION_REMOVE_ALIASES, false);
		aliasTable.addFocusListener(new FocusListener(){

			public void focusGained(FocusEvent e)
			{
				removeAlias.setEnabled(aliasTable.getSelection() != null && aliasTable.getSelection().length > 0);				
			}

			public void focusLost(FocusEvent e)
			{
				removeAlias.setEnabled(false);
				
			}
		});
		aliasTable.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				removeAlias.setEnabled(aliasTable.getSelection() != null && aliasTable.getSelection().length > 0);
			}
		});

			
		
		toolkit.createLabel(logicalClient, SMLMessages.editorLogicalTree);
		Tree logicalTree = toolkit.createTree(logicalClient, SWT.NONE);
		logicalTree.setLayout(new GridLayout());
		logicalTree.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		logicalRepresentation = new TreeViewer(logicalTree);
		logicalRepresentation.setContentProvider(new LogicalContentProvider());
		logicalRepresentation.setLabelProvider(new LogicalLabelProvider());
		logicalRepresentation.addSelectionChangedListener(new LogicalSelectionListener());
		makeSectionReady(logicalSection, logicalClient, SMLMessages.editorLogicalRepresentation);
		
		Section rawSection = createSection (sashForm);
		Composite rawSectionClient = toolkit.createComposite(rawSection);
		rawSectionClient.setLayout(new GridLayout());
		rawSectionClient.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		
		SMLIFEditor editor = (SMLIFEditor) getEditor();
		rawStyledText = editor.getExtendedToolkit().createStyledtext(rawSectionClient, null, SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL);
		rawStyledText.setLayout(new GridLayout());
		rawStyledText.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		rawStyledText.addLineStyleListener(new XMLLineStyler());
		rawStyledText.setFont(new Font(Display.getDefault(), "Courier New", 10, SWT.NORMAL)); //$NON-NLS-1$
		rawStyledText.setEditable(false);
		
		makeSectionReady(rawSection, rawSectionClient, SMLMessages.editorXMLRepresentation);
	}

	
	private void setTableProperties(TableViewer tableViewer)
	{
		TableLayout layout = new TableLayout();
		aliasTable.setLayout(layout);
		GridData aliasTableGD = new GridData(SWT.FILL, SWT.FILL, true, true);
		aliasTableGD.heightHint = 50;
		aliasTable.setLayoutData(aliasTableGD);
				
		ColumnPixelData columnLayout = new ColumnPixelData(300, true);
		layout.addColumnData(columnLayout);
		TableColumn column = new TableColumn(aliasTable, SWT.NONE, 0);
		column.setResizable(columnLayout.resizable);
		column.setText(SMLMessages.editorName);				
		
		/* Add the cell editors */
		tableViewer.setCellEditors(new CellEditor[]{new TextCellEditor(aliasTable)});
		tableViewer.setCellModifier(new ICellModifier()
		{
			public boolean canModify(Object element, String property)
			{
				return true;
			}

			public Object getValue(Object element, String property)
			{
				TableItem[] tableItem = aliasTable.getSelection();
				if (tableItem == null || tableItem.length != 1)
					return null;
			
				return tableItem[0].getText(0);
			}

			public void modify(Object element, String property, Object value)
			{				
				TableItem[] tableItem = aliasTable.getSelection();
				if (tableItem == null || tableItem.length != 1 || tableItem[0].getText().equals((String)value))
					return;
				
				handleOperation (OPERATION_MODIFY_ALIAS, new Object[]{value});
			}
			
		});
		
		final String ALIAS_COLUMN = "ALIAS_COLUMN"; //$NON-NLS-1$
		tableViewer.setColumnProperties(new String[]{ALIAS_COLUMN});
	}


	/**
	 * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetDefaultSelected(SelectionEvent e)
	{
		/* Doesn't need to be implemented */
	}


	/**
	 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetSelected(SelectionEvent e)
	{
		handleOperation(((Byte)e.widget.getData()).byteValue(), null);
	}	

	
	private boolean handleOperation(byte operation, Object[] arguments)
	{
		switch (operation)
		{
			case OPERATION_ADD_DOCUMENT:
			{
				/* Display the resource dialog */
				Tree documentTree = documentTreeViewer.getTree();
				TreeItem[] selection = documentTree.getSelection();
				if (selection == null || selection.length <= 0)
					return false;
				
				boolean genicsSelected = documentTree.indexOf(selection[0]) == 0;
				ResourceSelectionDialog resourceDialog = new ResourceSelectionDialog(SMLPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(), genicsSelected ? ISMLConstants.TYPE_GENIC : ISMLConstants.TYPE_PHENIC);
				resourceDialog.setMultiSelection(true);
				IFile[] selectedFile = resourceDialog.open();
				if (selectedFile == null || selectedFile.length <= 0)
					return false;

				SMLIFEditor editor = (SMLIFEditor)getEditor();
				Node rootNode = editor.retrieveChildNode(ISMLConstants.SMLIF_URI, genicsSelected ? ISMLConstants.GENICS_ELEMENT : ISMLConstants.PHENICS_ELEMENT);
				if (rootNode == null)
				{
					MessageDialog.openError(SMLPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(), 
							SMLMessages.errorAddingDocumentTitle, SMLMessages.errorAddingDocument0);
					return false;
				}					
					
				IFile file = null;
				
				/* For every selected item */
				try
				{
					for (int i = 0; i < selectedFile.length; i++)
					{
						file = selectedFile[i];
						Node documentNode = SMLCommonUtil.createElement(rootNode, ISMLConstants.SMLIF_URI, ISMLConstants.DOCUMENT_ELEMENT);
						Node dataNode = SMLCommonUtil.createElement(documentNode, ISMLConstants.SMLIF_URI, ISMLConstants.DATA_ELEMENT);		
						adoptNode (dataNode.getOwnerDocument(),
								dataNode,
								org.eclipse.cosmos.rm.validation.internal.common.SMLCommonUtil.domParseDocument(file.getContents(), false).getFirstChild());						
					}
				}
				catch (Exception e)
				{ 
					SMLCommonUtil.openErrorWithDetail(SMLMessages.errorAddingDocumentTitle, 
							NLS.bind(SMLMessages.errorAddingDocument1,
									file == null ? IValidationConstants.EMPTY_STRING : file.getName()), e);
					return false;
				} 
				
				documentTreeViewer.refresh();
				expandRootElement(genicsSelected);
				
				editor.markDirty();
				break;
			}
			case OPERATION_REMOVE_DOCUMENT:
			{
				Tree documentTree = documentTreeViewer.getTree();
				TreeItem[] selections = documentTree.getSelection();
				if (selections == null || selections.length <= 0)
					return false;
				
				if (!MessageDialog.openQuestion(SMLPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(),
						SMLMessages.editorRemoveDocumentTitle, SMLMessages.editorRemoveDocumentQuestion))
				{
					return false;
				}
				
				TreeItem parentItem = selections[0].getParentItem();
				int documentIndex = parentItem == null ? -1 : parentItem.indexOf(selections[0]);
				SMLIFEditor editor = (SMLIFEditor)getEditor();
				Node rootNode = documentIndex < 0 ? null : editor.retrieveChildNode(ISMLConstants.SMLIF_URI, documentTree.indexOf(parentItem) == 0 ? ISMLConstants.GENICS_ELEMENT : ISMLConstants.PHENICS_ELEMENT);
				Node[] rootChildren = null;
				if (rootNode == null || documentIndex < 0 || (rootChildren = SMLCommonUtil.retrieveChildElements(rootNode)) == null || documentIndex >= rootChildren.length)
				{
					MessageDialog.openError(SMLPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(),
						SMLMessages.errorRemovingDocumentTitle, SMLMessages.errorRemovingDocument0);
				}
				rootNode.removeChild(rootChildren[documentIndex]);
				
				documentTreeViewer.refresh();
				expandRootElement(documentTree.indexOf(parentItem) == 0);
				editor.markDirty();
				
				break;
			}
			case OPERATION_ADD_ALIASES:
			{
				Node aliases = SMLCommonUtil.nestedNodeRetrieval(input, new String[][]{
						new String[] {ISMLConstants.SMLIF_URI, ISMLConstants.DOC_INFO_ELEMENT},
						new String[] {ISMLConstants.SMLIF_URI, ISMLConstants.ALIASES_ELEMENT}
					}, true);
				
				Document ownerDocument = aliases.getOwnerDocument();				
				Node aliasElement = ownerDocument.createElementNS(ISMLConstants.SMLIF_URI, ISMLConstants.ALIAS_ELEMENT);
				aliasElement.setPrefix(aliases.getPrefix());
				aliases.appendChild(aliasElement);
				aliasElement.appendChild(ownerDocument.createTextNode(SMLMessages.editorAlias));
								
				TableItem tableItem = new TableItem (aliasTable, SWT.NONE);
				tableItem.setText(SMLMessages.editorAlias);				
						
				if (aliasTable.getItemCount() == 1)
					refreshContent();
				((SMLIFEditor)getEditor()).markDirty();
				break;
			}
			case OPERATION_REMOVE_ALIASES:
			{
				if (input == null)
					return false;
				Node aliases = SMLCommonUtil.nestedNodeRetrieval(input, new String[][]{
						new String[] {ISMLConstants.SMLIF_URI, ISMLConstants.DOC_INFO_ELEMENT},
						new String[] {ISMLConstants.SMLIF_URI, ISMLConstants.ALIASES_ELEMENT}
					});
				if (aliases == null)
				{ 
					MessageDialog.openError(SMLPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(), 
							SMLMessages.errorRemoving, SMLMessages.errorRemovingAlias1);
					return false;
				}
				TableItem[] selection = aliasTable.getSelection();
				if (selection.length != 1)
				{
					MessageDialog.openError(SMLPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(), 
							SMLMessages.errorRemoving, SMLMessages.errorRemovingAlias2);
					return false;
				}
				int inx = aliasTable.indexOf(selection[0]);
				NodeList aliasList = aliases.getChildNodes();
				int counter = -1;
				Node currentAlias = null;
				for (int i = 0, aliasCount = aliasList.getLength(); i < aliasCount && counter != inx; i++)
				{
					currentAlias = aliasList.item(i);
					if (currentAlias.getNodeType() == Node.ELEMENT_NODE)
						counter++;
				}
				if (counter != inx)
				{
					MessageDialog.openError(SMLPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(), 
							SMLMessages.errorRemoving, SMLMessages.errorRemovingAlias1);
					return false;
				}
				currentAlias.getParentNode().removeChild(currentAlias);
				aliasTable.remove(inx);
				
				if (inx == 0)
					refreshContent();
				((SMLIFEditor)getEditor()).markDirty();			
				break;
			}
			case OPERATION_MODIFY_ALIAS:
			{
				Node aliases = SMLCommonUtil.nestedNodeRetrieval(input, new String[][]{
						new String[] {ISMLConstants.SMLIF_URI, ISMLConstants.DOC_INFO_ELEMENT},
						new String[] {ISMLConstants.SMLIF_URI, ISMLConstants.ALIASES_ELEMENT}});
				
				TableItem[] selection = aliasTable.getSelection();
				int selectionInx = selection == null || selection.length != 1 ? -1 : aliasTable.indexOf(selection[0]);
				Node[] aliasList = SMLCommonUtil.retrieveChildElements(aliases);
				if (aliases == null || selectionInx < 0 || selectionInx >= aliasList.length)
				{
					MessageDialog.openError(SMLPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(), 
							SMLMessages.errorModifyingAliasTitle, SMLMessages.errorModifyingAlias0);
					if (selectionInx >= 0)
						aliasTable.remove(selectionInx);
					return false;
				}
				
				NodeList aliasChildNodes = aliasList[selectionInx].getChildNodes();
				for (int i = 0, aliasChildNodeCount = aliasChildNodes.getLength(); i < aliasChildNodeCount; i++)
				{
					aliasList[selectionInx].removeChild(aliasChildNodes.item(i));
				}
				aliasList[selectionInx].appendChild(aliasList[selectionInx].getOwnerDocument().createTextNode((String)arguments[0]));
				
				if (selectionInx == 0)
					refreshContent();
				((SMLIFEditor)getEditor()).markDirty();
				
				break;
			}
		}
		
		return true;
	}


	private void refreshContent()
	{
		Tree documentTree = documentTreeViewer.getTree();
		TreeItem[] selection = documentTree.getSelection();
		if (selection != null && selection.length > 0)
		{
			int itemInx = selection[0].getParentItem().indexOf(selection[0]);
			boolean genicsSelected = documentTree.indexOf(selection[0].getParentItem()) == 0;
			documentTreeViewer.refresh();				
			expandRootElement(genicsSelected);
			
			Event event = new Event();
			event.type = SWT.Selection;
			event.widget = documentTree;
			event.item= documentTree.getItem(genicsSelected ? 0 : 1).getItem(itemInx);
			documentTree.setSelection((TreeItem)event.item);
			documentTree.notifyListeners(event.type, event);				
		}
	}

	private void expandRootElement(boolean genicsSelected)
	{		
		Tree documentTree = documentTreeViewer.getTree();		
		int expandedInx = genicsSelected ? 0 : 1;
		if (expandedInx < documentTree.getItemCount())
		{
			Event event = new Event();
			event.type = SWT.Expand;
			event.widget = documentTree;
			event.item= documentTree.getItem(expandedInx);
			documentTree.getItem(expandedInx).setExpanded(true);
			documentTree.notifyListeners(event.type, event);
		}
	}

	private void adoptNode(Document document, Node parent, Node node)
	{
		switch (node.getNodeType())
		{
			case Node.TEXT_NODE:
			{
				String value = node.getNodeValue();				
				if (value == null)
					return;
				
				parent.appendChild(document.createTextNode(value));
				break;
			}			
					
			case Node.COMMENT_NODE:
			{
				String value = node.getNodeValue();				
				if (value == null)
					return;
				
				parent.appendChild(document.createComment(value));
			}			
			
			case Node.ELEMENT_NODE:
			{
				Node clonedNode = document.createElementNS(node.getNamespaceURI(), node.getNodeName());
				clonedNode.setPrefix(node.getPrefix());
				NamedNodeMap attributes = node.getAttributes();
								
				if (attributes != null)
				{
					for (int i = 0, attCount = attributes.getLength(); i < attCount; i++)
					{
						adoptNode (document, clonedNode, attributes.item(i));
					}
				}
				NodeList children = node.getChildNodes();
				if (children != null)
				{
					for (int i = 0, childCount = children.getLength(); i < childCount; i++)
					{
						adoptNode (document, clonedNode, children.item(i));
					}
				}
				
				parent.appendChild(clonedNode);				
				break;
			}	
			case Node.ATTRIBUTE_NODE:
			{
				Node clonedNode = document.createAttributeNS(node.getNamespaceURI(), node.getNodeName());
				clonedNode.setNodeValue(node.getNodeValue());
				parent.getAttributes().setNamedItemNS(clonedNode);
				
				break;
			}			
		}
	}	
	
	private boolean isGenicOrPhenic(Object element)
	{			
		Node node = null;
		return 	element instanceof DocumentTreeElement && 
				(node = ((DocumentTreeElement)element).getNode()) != null &&
				ISMLConstants.SMLIF_URI.equals(node.getNamespaceURI()) &&
				
				(ISMLConstants.GENICS_ELEMENT.equals(node.getLocalName()) ||
				ISMLConstants.PHENICS_ELEMENT.equals(node.getLocalName()));					
	}
	
	
	/**
	 * Invoked when an element is selected under the document
	 * tree.
	 */
	public void selectionChanged(SelectionChangedEvent event)
	{
		ISelection selection = event.getSelection();
		StructuredSelection structuredSelection = null;
		if (!selection.isEmpty() && selection instanceof StructuredSelection && 
			(structuredSelection = (StructuredSelection)selection).size() > 0)
		{
			DocumentTreeElement input = (DocumentTreeElement)structuredSelection.getFirstElement();
			setControlEnabble(!input.isRootElement());
			setInput(input.isRootElement() ? null : input.getNode());
		}
		else
		{
			setInput(null);
		}
	}

	
	private void setControlEnabble(boolean enable)
	{
		addAlias.setEnabled(enable);
		rawStyledText.setEnabled(enable);
	}


	private void setInput(Object input)
	{		
		/* The alias table */
		Node aliasesNode = SMLCommonUtil.nestedNodeRetrieval((Node)input, new String[][] {
				new String[] {ISMLConstants.SMLIF_URI, ISMLConstants.DOC_INFO_ELEMENT},
				new String[] {ISMLConstants.SMLIF_URI, ISMLConstants.ALIASES_ELEMENT}
				});
		aliasTable.removeAll();
		populateTableAlias(aliasesNode);
		
		/* The logical tree representation */
		this.input = (Node)input;
		logicalRepresentation.setInput(input);
		
		/* The raw text representation */
		indices.clear();
		Node[] node = null;
		if (input == null || (node = extractDocumentContent((Node)input)).length <= 0)
		{
			rawStyledText.setText(IValidationConstants.EMPTY_STRING);
			return;
		}		
		
		StringBuffer sb = new StringBuffer();
		SMLCommonUtil.serialzeNode (sb, node[0], 0, "0", indices); //$NON-NLS-1$
		rawStyledText.setText(sb.toString());
	}


	private void populateTableAlias(Node aliasesNode)
	{
		removeAlias.setEnabled(false);
		NodeList aliases = null;
		if (aliasesNode == null || (aliases = aliasesNode.getChildNodes()) == null || 
			aliases.getLength() <= 0)
			return;
			
		for (int i = 0, aliasCount = aliases.getLength(); i < aliasCount; i++)
		{
			Node currentAlias = aliases.item(i);
			if (currentAlias.getNodeType() == Node.TEXT_NODE)
				continue;
			
			String alias = SMLCommonUtil.retrieveTextNode(currentAlias);
			
			if (alias != null && alias.length() > 0)
			{
				TableItem tableItem = new TableItem (aliasTable, SWT.NONE);
				tableItem.setText(alias);
			}
		}
	}


	private Node[] extractDocumentContent(Node documentNode)
	{
		Node dataNode = SMLCommonUtil.retrieveChildNode((Node)documentNode, 
				ISMLConstants.SMLIF_URI, ISMLConstants.DATA_ELEMENT);
		Node[] elements = SMLCommonUtil.retrieveChildElements(dataNode);
		if (elements.length <= 0)
		{
			dataNode = SMLCommonUtil.retrieveChildNode((Node)input, 
					ISMLConstants.SMLIF_URI, ISMLConstants.LOCATOR_ELEMENT);
			return dataNode == null ? new Node[0] : new Node[]{dataNode};
		}
		
		return elements;
	}
	
	
	/**
	 * This is the content provider for the document tree
	 * 
	 * @author Ali Mehregani
	 */
	private class DocumentContentProvider implements ITreeContentProvider
	{
		
		/**
		 * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
		 */
		public Object[] getChildren(Object parent)
		{
			Node node = ((DocumentTreeElement)parent).getNode();
			if (node == null)
				return new Object[0];
			
			List children = new ArrayList();
			NodeList nodeChildren = node.getChildNodes();
			for (int i = 0, childCount = nodeChildren.getLength(); i < childCount; i++)
			{
				Node currentChildNode = nodeChildren.item(i);
				if (currentChildNode.getNodeType() == Node.ELEMENT_NODE)
				{
					DocumentTreeElement element = new DocumentTreeElement();
					element.setNode(nodeChildren.item(i));
					children.add(element);
				}
			}				
			
			return children.toArray();
		}

		/**
		 * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
		 */
		public Object getParent(Object element)
		{
			return null;
		}

		/**
		 * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
		 */
		public boolean hasChildren(Object element)
		{			
			return isGenicOrPhenic(element);
		}

		/**
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
		 */
		public Object[] getElements(Object inputElement)
		{
			SMLIFEditor editor = (SMLIFEditor)getEditor();
			
			List elements = new ArrayList();
			addElement (elements, 
					editor.retrieveChildNode(ISMLConstants.SMLIF_URI, ISMLConstants.GENICS_ELEMENT), 
					SMLMessages.editorGenics); 
			addElement (elements, 
					editor.retrieveChildNode(ISMLConstants.SMLIF_URI, ISMLConstants.PHENICS_ELEMENT),
					SMLMessages.editorPhenics);
					

			return elements.toArray();
		}

		private void addElement(List elements, Node node, String text)
		{
			if (node == null)
				return;
			
			DocumentTreeElement element = new DocumentTreeElement(true);
			element.setText(text);
			element.setNode(node);
			
			elements.add(element);
		}

		public void dispose()
		{
		}

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{		
		}
	
	}
	
	
	/**
	 * The label provider for the document tree
	 * 
	 * @author Ali Mehregani
	 */
	private class DocumentLabelProvider implements ILabelProvider
	{
		/**
		 * @see org.eclipse.jface.viewers.ILabelProvider#getImage(java.lang.Object)
		 */
		public Image getImage(Object element)
		{			
			if (isGenicOrPhenic(element))
			{
				return ISMLConstants.GENICS_ELEMENT.equals(((DocumentTreeElement)element).getNode().getLocalName()) ?
						SMLImages.INSTANCE.getImage(SMLImages.GENICS) :
						SMLImages.INSTANCE.getImage(SMLImages.PHENICS);
			}
			else
			{
				return SMLImages.INSTANCE.getImage(SMLImages.DOCUMENTS);
			}
		}

		/**
		 * @see org.eclipse.jface.viewers.ILabelProvider#getText(java.lang.Object)
		 */
		public String getText(Object element)
		{
			DocumentTreeElement treeElement = (DocumentTreeElement)element;
			if (treeElement.getText() != null)
				return treeElement.getText();
			
			return treeElement.retrieveLabel();			
		}

		/**
		 * @see org.eclipse.jface.viewers.IBaseLabelProvider#addListener(org.eclipse.jface.viewers.ILabelProviderListener)
		 */
		public void addListener(ILabelProviderListener listener)
		{
			
		}

		/**
		 * @see org.eclipse.jface.viewers.IBaseLabelProvider#dispose()
		 */
		public void dispose()
		{
			
		}

		/**
		 * @see org.eclipse.jface.viewers.IBaseLabelProvider#isLabelProperty(java.lang.Object, java.lang.String)
		 */
		public boolean isLabelProperty(Object element, String property)
		{
			return false;
		}

		/**
		 * @see org.eclipse.jface.viewers.IBaseLabelProvider#removeListener(org.eclipse.jface.viewers.ILabelProviderListener)
		 */
		public void removeListener(ILabelProviderListener listener)
		{
		}		
	}
	
	
	/**
	 * Represents each element that appears under the document tree
	 * 
	 * @author Ali Mehregani
	 */
	private static class DocumentTreeElement
	{
		/**
		 * The associated node
		 */
		private Node node;
		
		/**
		 * The associated text
		 */
		private String text;

		/**
		 * Indicates whether this is a root element
		 */
		private boolean rootElement;
				
		public DocumentTreeElement()
		{
			
		}
		
		public DocumentTreeElement(boolean rootElement)
		{
			this.rootElement = rootElement;
		}
		
		/**
		 * The label of an element representing an SML document is either
		 * the first alias of the document or the first element name of
		 * the document.
		 *  
		 * @return The label for a tree element representing an SML document
		 */
		public String retrieveLabel()
		{
			if (node == null)
				return null;
			
			Node alias = SMLCommonUtil.nestedNodeRetrieval (node, new String[][]{
				new String[] {ISMLConstants.SMLIF_URI, ISMLConstants.DOC_INFO_ELEMENT},
				new String[] {ISMLConstants.SMLIF_URI, ISMLConstants.ALIASES_ELEMENT},
				new String[] {ISMLConstants.SMLIF_URI, ISMLConstants.ALIAS_ELEMENT}
			});
			
			text = SMLCommonUtil.retrieveTextNode(alias);
			if (text != null && text.length() > 0)
				return text;
			
			Node childNode = SMLCommonUtil.retrieveChildNode(node, ISMLConstants.SMLIF_URI, ISMLConstants.DATA_ELEMENT);
			if (childNode == null)
			{
				childNode = SMLCommonUtil.retrieveChildNode(node, ISMLConstants.SMLIF_URI, ISMLConstants.LOCATOR_ELEMENT);
				if (childNode == null)
				{
					return SMLMessages.commonUnknown;
				}
				Node reference = childNode.getAttributes().getNamedItemNS(ISMLConstants.XLINK_URI, ISMLConstants.HREF_ATTRIBUTE);
				return reference == null ? SMLMessages.commonUnknown : SMLMessages.editorRemoteDoc + reference.getNodeValue();
			}
			else
			{
				NodeList children = childNode.getChildNodes();
				String text = retrieveElementName(children);
				return text == null ? SMLMessages.commonUnknown : text;
			}
		}

		private String retrieveElementName(NodeList children)
		{
			for (int i = 0, childCount = children.getLength(); i < childCount; i++)
			{
				Node node = children.item(i);
				if (node.getNodeType() != Node.TEXT_NODE && node.getLocalName() != null)
					return node.getLocalName();
			}
			
			return null;
		}

		/**
		 * @return the node
		 */
		public Node getNode()
		{
			return node;
		}
		
		/**
		 * @param node the node to set
		 */
		public void setNode(Node node)
		{
			this.node = node;
		}


		/**
		 * @return the text
		 */
		public String getText()
		{
			return text;
		}


		/**
		 * @param text the text to set
		 */
		public void setText(String text)
		{
			this.text = text;
		}		
		
		/**
		 * @return A flag indicating whether this is a root element
		 */
		public boolean isRootElement()
		{
			return rootElement;
		}
	}
	
	
	/**
	 * This content provider is used to provide the content for the logical
	 * tree representation of a document
	 * 
	 * @author Ali Mehregani
	 */
	private class LogicalContentProvider implements ITreeContentProvider
	{

		/**
		 * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
		 */
		public Object[] getChildren(Object parent)
		{
			return SMLCommonUtil.retrieveChildElements((Node)parent);
		}

		/**
		 * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
		 */
		public Object getParent(Object element)
		{
			return null;
		}

		
		/**
		 * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
		 */
		public boolean hasChildren(Object element)
		{
			return SMLCommonUtil.retrieveChildElements((Node)element).length > 0;
		}

		/**
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
		 */
		public Object[] getElements(Object input)
		{			
			return extractDocumentContent((Node)input);
		}

		/**
		 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
		 */
		public void dispose()
		{
		}

		/**
		 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
		 */
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}		
	}
	
	
	/**
	 * The label provider for the logical tree representaion of a document.
	 * 
	 * @author Ali Mehregani
	 */
	private class LogicalLabelProvider implements ILabelProvider
	{
		/**
		 * @see org.eclipse.jface.viewers.ILabelProvider#getImage(java.lang.Object)
		 */
		public Image getImage(Object element)
		{
			return SMLImages.INSTANCE.getImage(SMLImages.ELEMENT);
		}

		/**
		 * @see org.eclipse.jface.viewers.ILabelProvider#getText(java.lang.Object)
		 */
		public String getText(Object element)
		{
			return ((Node)element).getNodeName();
		}

		/**
		 * @see org.eclipse.jface.viewers.IBaseLabelProvider#addListener(org.eclipse.jface.viewers.ILabelProviderListener)
		 */
		public void addListener(ILabelProviderListener listener)
		{
		}

		/**
		 * @see org.eclipse.jface.viewers.IBaseLabelProvider#dispose()
		 */
		public void dispose()
		{
		}

		/** 
		 * @see org.eclipse.jface.viewers.IBaseLabelProvider#isLabelProperty(java.lang.Object, java.lang.String)
		 */
		public boolean isLabelProperty(Object element, String property)
		{
			return false;
		}

		/**
		 * @see org.eclipse.jface.viewers.IBaseLabelProvider#removeListener(org.eclipse.jface.viewers.ILabelProviderListener)
		 */
		public void removeListener(ILabelProviderListener listener)
		{
		}
		
	}

	
	/**
	 * The selection listener for the logical tree representation of
	 * a document.  This is used to select the corresponding fragment under
	 * the XML representation text box
	 * 
	 * @author Ali Mehregani
	 */
	private class LogicalSelectionListener implements ISelectionChangedListener
	{
		/**
		 * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(org.eclipse.jface.viewers.SelectionChangedEvent)
		 */
		public void selectionChanged(SelectionChangedEvent event)
		{
			ISelection selection = event.getSelection();
			if (!(selection instanceof StructuredSelection) || selection.isEmpty())
			{
				rawStyledText.setSelection(0,0);
				return;
			}
					
			String index = findSelectionIndex (null, logicalRepresentation.getTree().getSelection()[0]);
			int[] elementIndices = (int[])indices.get(index);
			if (elementIndices == null)
			{
				rawStyledText.setSelection(0,0);
				return;
			}
			
			rawStyledText.setSelection(elementIndices[0], elementIndices[1]);
		}

		
		private String findSelectionIndex(String index, TreeItem item)
		{
			TreeItem parentItem = item.getParentItem();
			if (parentItem != null)
			{
				String inx = String.valueOf(parentItem.indexOf(item));
				if (index != null && index.length() > 0)
					inx += "," + index; //$NON-NLS-1$
				
				return findSelectionIndex(inx, parentItem);
			}
			return "0" + (index == null || index.length() <= 0 ? IValidationConstants.EMPTY_STRING : "," + index); //$NON-NLS-1$ //$NON-NLS-2$
		}		
	}
}
