/**********************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.smlif.internal.editor;


import org.eclipse.cosmos.rm.smlif.internal.SMLImages;
import org.eclipse.cosmos.rm.smlif.internal.SMLPlugin;
import org.eclipse.cosmos.rm.smlif.internal.actions.ValidateResourceAction;
import org.eclipse.cosmos.rm.smlif.internal.common.SMLCommonUtil;
import org.eclipse.cosmos.rm.smlif.internal.common.SMLMessages;
import org.eclipse.cosmos.rm.validation.internal.common.ISMLConstants;
import org.eclipse.cosmos.rm.validation.internal.common.IValidationConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.forms.events.HyperlinkEvent;
import org.eclipse.ui.forms.events.IHyperlinkListener;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Hyperlink;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.part.FileEditorInput;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * The overview page of the SML-IF editor
 * 
 * @author Ali Mehregani
 */
public class OverviewPage extends AbstractFormPage
{
	/**
	 * The id of this page
	 */
	public static final String ID = SMLPlugin.PLUGIN_ID + ".OverviewPage";

	/**
	 * The description text field
	 */
	private Text descriptionText;
	
	/**
	 * The name text field
	 */
	private Text nameText;
	
	/**
	 * The display name text field
	 */
	private Text displayNameText;
	
	/**
	 * The base uri text field
	 */
	private Text baseURIText;
	
	/**
	 * Used to disable the modify listeners of the text fields
	 */
	private boolean disableListener;
	
	/**
	 * Constructor
	 * 
	 * @param editor The form editor
	 */
	public OverviewPage(FormEditor editor)
	{
		super(editor, ID, SMLMessages.editorOverviewTitle);
	}
	
	
	/**
	 * @see org.eclipse.cosmos.rm.smlif.internal.editor.AbstractFormPage#createLeftColumn(org.eclipse.swt.widgets.Composite)
	 */
	protected void createLeftColumn(Composite parent)
	{
		getManagedForm().addPart(((SMLIFEditor)getEditor()).getSmlifFormPart());
		FormToolkit toolkit = getManagedForm().getToolkit();
		Section section = createSection(parent);
		
		Composite sectionClient = toolkit.createComposite(section);
		GridLayout sectionClientGL = new GridLayout(2, false);
		sectionClientGL.marginTop = 10;
		sectionClient.setLayout(sectionClientGL);
		sectionClient.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		nameText = createTextField(sectionClient, SMLMessages.editorName, null, this);
		displayNameText = createTextField(sectionClient, SMLMessages.editorDisplayName, null, this);
		baseURIText = createTextField(sectionClient, SMLMessages.editorBaseURI, null, this);
		descriptionText = createTextField(sectionClient, SMLMessages.editorDescription, true, null, this);
		
		makeSectionReady (section, sectionClient, SMLMessages.editorGeneralInformation);
	}



	/**
	 * @see org.eclipse.cosmos.rm.smlif.internal.editor.AbstractFormPage#createRightColumn(org.eclipse.swt.widgets.Composite)
	 */
	protected void createRightColumn(Composite parent)
	{
		FormToolkit toolkit = getManagedForm().getToolkit();
		Section section = createSection(parent);
		
		Composite sectionClient = toolkit.createComposite(section);
		GridLayout gl = new GridLayout(2, false);
		gl.marginTop = 10;
		sectionClient.setLayout(gl);
		sectionClient.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		
		Label linkImage = toolkit.createLabel(sectionClient, IValidationConstants.EMPTY_STRING);
		linkImage.setImage(SMLImages.INSTANCE.getImage(SMLImages.VALIDATE));
		
		Hyperlink validateLink = toolkit.createHyperlink(sectionClient, SMLMessages.editorValidateDocument, SWT.WRAP);
		FontData[] fontData = validateLink.getFont().getFontData();
		for (int i = 0; i < fontData.length; i++)
		{
			fontData[i].setStyle(fontData[i].getStyle() | SWT.BOLD);
		}
		validateLink.setFont(new Font(null,fontData));	
		
		validateLink.addHyperlinkListener(new IHyperlinkListener()
		{
			public void linkActivated(HyperlinkEvent e)
			{
				IEditorInput input = getEditorInput();
				if (input == null)
				{
					MessageDialog.openError(SMLPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(), 
							SMLMessages.errorNoInputTitle, SMLMessages.errorNoInput);
					return;
				}
				
				ValidateResourceAction validateAction = new ValidateResourceAction();
				validateAction.selectionChanged(null, new StructuredSelection(((FileEditorInput)input).getFile()));
				validateAction.run(null);
			}

			public void linkEntered(HyperlinkEvent e)
			{
			}

			public void linkExited(HyperlinkEvent e)
			{
			}
		});
		

		makeSectionReady (section, sectionClient, SMLMessages.editorValidateDocument);
	}
	
	public void setActive(boolean active)
	{	
		super.setActive(active);
		
		if (!active)
			return;
		
		disableListener = true;
		
		/* Populate the fields */
		SMLIFEditor editor = (SMLIFEditor)getEditor();		
		
		String field = editor.retrieveField(SMLIFEditor.TYPE_IDENTITY, ISMLConstants.NAME_ELEMENT);
		nameText.setText(field == null ? IValidationConstants.EMPTY_STRING : field);
		
		field = editor.retrieveField(SMLIFEditor.TYPE_IDENTITY, ISMLConstants.DISPLAY_NAME_ELEMENT);
		displayNameText.setText(field == null ? IValidationConstants.EMPTY_STRING : field);
		
		field = editor.retrieveField(SMLIFEditor.TYPE_IDENTITY, ISMLConstants.BASE_URI_ELEMENT);
		baseURIText.setText(field == null ? IValidationConstants.EMPTY_STRING : field);
		
		field = editor.retrieveField(SMLIFEditor.TYPE_IDENTITY, ISMLConstants.DESCRIPTION_ELEMENT);
		descriptionText.setText(field == null ? IValidationConstants.EMPTY_STRING : field);
		
		disableListener = false;
	}
	
	public void modifyText(ModifyEvent e)
	{		
		if (disableListener)
			return;
		
		SMLIFEditor editor = (SMLIFEditor)getEditor();
		Node identityNode = SMLCommonUtil.nestedNodeRetrieval(editor.getDocumentNode(), new String[][]{new String[] {ISMLConstants.SMLIF_URI, ISMLConstants.IDENTITY_ELEMENT}}, true);				
		modifyTextNode(editor, identityNode, 
					e.widget == nameText ? 			ISMLConstants.NAME_ELEMENT :			/* The name field */			 
					e.widget == displayNameText ? 	ISMLConstants.DISPLAY_NAME_ELEMENT : 	/* Display name */
					e.widget == baseURIText ? 		ISMLConstants.BASE_URI_ELEMENT : 		/* The base URI */
													ISMLConstants.DESCRIPTION_ELEMENT, 		/* Description */
					((Text)e.widget).getText());
		
		editor.markDirty();		
	}


	private void modifyTextNode(SMLIFEditor editor, Node node, String elementName, String newValue)
	{
		Node childNode = SMLCommonUtil.nestedNodeRetrieval(node, new String[][]{new String[] {ISMLConstants.SMLIF_URI, elementName}}, true);
		
		/* Remove all children of the node */
		NodeList children = childNode.getChildNodes();
		for (int i = 0, childCount = children.getLength(); i < childCount; i++)
		{
			childNode.removeChild(children.item(i));
		}
		
		childNode.appendChild(childNode.getOwnerDocument().createTextNode(newValue == null ? IValidationConstants.EMPTY_STRING : newValue));
	}

}
