package org.eclipse.cosmos.rm.smlif.internal.editor;


import java.io.IOException;
import java.io.InputStream;

import javax.xml.parsers.ParserConfigurationException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.cosmos.rm.smlif.internal.SMLPlugin;
import org.eclipse.cosmos.rm.smlif.internal.common.SMLCommonUtil;
import org.eclipse.cosmos.rm.smlif.internal.common.SMLMessages;
import org.eclipse.cosmos.rm.validation.internal.common.ISMLConstants;
import org.eclipse.cosmos.rm.validation.internal.common.IValidationConstants;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.forms.editor.FormEditor;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * The SML-IF editor part used to create three sub-pages: overview,
 * documents, and sml-if
 * 
 * @author Ali Mehregani
 */
public class SMLIFEditor extends FormEditor implements IResourceChangeListener
{

	public static final int TYPE_IDENTITY = 0;

	/**
	 * The overview page
	 */
	private OverviewPage overviewPage;

	/**
	 * The documents pages
	 */
	private DocumentsPage documentsPage;

	/**
	 * The sml-if page
	 */
	private SMLIFPage smlifPage;

	
	/**
	 * The root DOM node of the SML-IF file selected 
	 */
	private Node rootNode;

	/**
	 * The identity node
	 */
	private Node identityNode;

	
	/**
	 * Indicates whether an error has been reported
	 */
	private boolean reportedError;
	
	/**
	 * An extended toolkit
	 */
	private SMLFormToolkit extendedToolkit;
	
	/**
	 * The form pages contained by this editor
	 */
	private AbstractFormPage[] formPages;

	/**
	 * The DOM document representing the SML-IF content
	 */
	private Document document;
	
	/**
	 * The form part shared amongst the form pages
	 */
	private SMLIFFormPart smlifFormPart;
	
	
	/**
	 * Create the multi-page editor for SML-IF documents
	 */
	public SMLIFEditor() 
	{
		super();
		ResourcesPlugin.getWorkspace().addResourceChangeListener(this);
		extendedToolkit = new SMLFormToolkit (SMLPlugin.getDefault().getWorkbench().getDisplay());
		smlifFormPart = new SMLIFFormPart(this);
	}
	
	
	/**
	 * @see org.eclipse.ui.part.MultiPageEditorPart#init(org.eclipse.ui.IEditorSite, org.eclipse.ui.IEditorInput)
	 */
	public void init(IEditorSite site, IEditorInput input) throws PartInitException
	{
		if (!(input instanceof IFileEditorInput))
			throw new PartInitException (SMLMessages.errorEditorWrongInput);
			
		super.init(site, input);
		setPartName(((IFileEditorInput)input).getFile().getName());
	}

	/**
	 * @see org.eclipse.ui.forms.editor.FormEditor#addPages()
	 */
	protected void addPages() 
	{
		overviewPage = new OverviewPage(this);
		documentsPage = new DocumentsPage(this);
		smlifPage = new SMLIFPage(this);
		formPages = new AbstractFormPage[]{overviewPage, documentsPage, smlifPage};
		
		try
		{
			for (int i = 0; i < formPages.length; i++)
			{
				addPage(formPages[i], getEditorInput());
			}
			
			getSite().setSelectionProvider(null);
			setActivePage(OverviewPage.ID);
		} 
		catch (PartInitException e)
		{
			SMLCommonUtil.openErrorWithDetail(SMLMessages.errorEditorTitle, SMLMessages.errorEditorPages, e);
		}
		
		
	}
	
	
	/**
	 * @see org.eclipse.ui.part.MultiPageEditorPart#dispose()
	 */
	public void dispose() 
	{
		ResourcesPlugin.getWorkspace().removeResourceChangeListener(this);
		super.dispose();
	}
	
	
	/**
	 * @see org.eclipse.ui.part.EditorPart#doSave(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void doSave(IProgressMonitor monitor) 
	{
		smlifFormPart.commit(true);		
		if (!isDirty())
			editorDirtyStateChanged();
	}
	
	
	/**
	 * @see org.eclipse.ui.part.EditorPart#doSaveAs()
	 */
	public void doSaveAs() 
	{
		smlifPage.doSaveAs();
		setInput(smlifPage.getEditorInput());
	}
	

	/**
	 * @see org.eclipse.ui.part.EditorPart#isSaveAsAllowed()
	 */
	public boolean isSaveAsAllowed() 
	{
		return true;
	}
	
	
	/**
	 * Closes all project files on project close.
	 */
	public void resourceChanged(final IResourceChangeEvent event)
	{
		if (event.getType() != IResourceChangeEvent.PRE_CLOSE)
			return;
		
		Runnable closeEditors = new Runnable()
		{
			public void run()
			{
				IWorkbenchPage[] pages = getSite().getWorkbenchWindow().getPages();
				for (int i = 0; i<pages.length; i++)
				{
					if(((IFileEditorInput)getEditorInput()).getFile().getProject().equals(event.getResource()))
					{
						IEditorPart editorPart = pages[i].findEditor(getEditorInput());
						pages[i].closeEditor(editorPart,true);
					}
				}
			}    
		};
		
		Display.getDefault().asyncExec(closeEditors);
	}


	public String retrieveField(int type, String name)
	{
		Node rootDocumentNode = getDocumentNode();
		
		/* The identity type */
		if (type == TYPE_IDENTITY)
		{
			identityNode = identityNode == null ? SMLCommonUtil.retrieveChildNode(rootDocumentNode, ISMLConstants.SMLIF_URI, ISMLConstants.IDENTITY_ELEMENT) : identityNode;			
			return SMLCommonUtil.retrieveTextNode(SMLCommonUtil.retrieveChildNode(identityNode, ISMLConstants.SMLIF_URI, name));
		}
		
		return null;
	}
	
	/**
	 * Equivalent to calling retrieveChildNode(Node parent, String uri, String localName),
	 * where parent is the root node of the associated document
	 * 
	 * @param uri The URI
	 * @param localName The localName of the element to be retrieved
	 */
	public Node retrieveChildNode(String uri, String localName)
	{
		return SMLCommonUtil.retrieveChildNode(getDocumentNode(), uri, localName);
	}
	
	/**
	 * Parse and return the DOM document of the SML-IF document
	 * 
	 * @return The DOM document node of the SML-IF file
	 */
	protected Node getDocumentNode()
	{
		if (rootNode != null)
			return rootNode;
		
		if (getEditorInput() == null)
			return null;
		
		IFile selectedFile = ((IFileEditorInput)getEditorInput()).getFile();
		if (selectedFile == null)
			return null;
		
		InputStream is = null;
		Exception exception = null;
		int lineNumber = -1;
		try
		{
			is = selectedFile.getContents();
			document = org.eclipse.cosmos.rm.validation.internal.common.SMLCommonUtil.domParseDocument(is, false);
			NodeList children = document.getChildNodes();			
			for (int i = 0, childCount = children.getLength(); i < childCount; i++)
			{
				if (children.item(i).getNodeType() == Node.ELEMENT_NODE)
				{
					rootNode = children.item(i);
					break;
				}
			}
			if (rootNode == null || 
				!ISMLConstants.SMLIF_URI.equals(rootNode.getNamespaceURI()) ||
				!ISMLConstants.MODEL_ELEMENT.equals(rootNode.getLocalName()))
			{
				rootNode = null;				
				throw new SAXException (SMLMessages.errorWrongRootNode);
			}
						
			return rootNode;
			
		} 
		catch (CoreException e)
		{
			exception = e;
		} 
		catch (ParserConfigurationException e)
		{
			exception = e;
		} 
		catch (SAXParseException e)
		{
			exception = e;
			lineNumber = e.getLineNumber();
		}
		catch (SAXException e)
		{
			exception = e;
		} 
		catch (IOException e)
		{
			exception = e;
		}
		finally
		{			
			try
			{
				if (is != null)
					is.close();
			} catch (IOException e)
			{
				/* Ignore the exception */
			}
		}
		
		if (exception != null && !reportedError)
		{
			reportedError = true;
			SMLCommonUtil.openErrorWithDetail(SMLMessages.errorOpeningTitle, 
					SMLMessages.errorOpening + (lineNumber >= 0 ? IValidationConstants.LINE_SEPARATOR + 
					NLS.bind(SMLMessages.errorOpeningFileLine, String.valueOf(lineNumber)) : IValidationConstants.EMPTY_STRING), 
					exception);
			
			for (int i = 0; i < formPages.length; i++)
			{
				formPages[i].setEnabled(false);
			}
		}
		
		return null;
	}

	public Document getDocument()
	{
		if (document == null)
			getDocumentNode();
		return document;
	}
	
	
	/**
	 * @return the extendedToolkit
	 */
	public SMLFormToolkit getExtendedToolkit()
	{
		return extendedToolkit;
	}


	/**
	 * @return the smlifFormPart
	 */
	public SMLIFFormPart getSmlifFormPart()
	{
		return smlifFormPart;
	}

	/**
	 * Marks this editor as dirty
	 */
	public void markDirty()
	{			
		boolean markEditorDirty = false;
		markEditorDirty = !isDirty();
		
		smlifFormPart.markDirty();
		if (markEditorDirty)
			editorDirtyStateChanged();
	}

}
