/**********************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.smlif.internal.importing;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.cosmos.rm.smlif.internal.common.SMLMessages;
import org.eclipse.cosmos.rm.validation.internal.common.ISMLConstants;
import org.eclipse.cosmos.rm.validation.internal.common.SMLCommonUtil;
import org.eclipse.cosmos.rm.validation.internal.common.SMLCommonUtil.RemoteRetrievalException;
import org.eclipse.osgi.util.NLS;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * SAX Handler for parsing an SML-IF file for genic and phenic model units
 * 
 * @author David Whiteman
 */
public class SMLIFFileHandler extends DefaultHandler {

private String baseURI;
private boolean buildingModelUnit;
private ModelUnit currentModel;
private boolean inAliases = false;
private boolean inBaseURI;
private boolean inData = false;
//private boolean inLocator;
private boolean inGenics = false;
//private boolean inPhenics = false;
private List modelUnits = new ArrayList();

public void characters(char[] ch, int start, int length) throws SAXException {
	if (inData) {
		writeToModelUnitFile(new String(ch, start, length));
	}
	if (inAliases) {
		// Since we are in the "aliases" section, we can append the input
		// string to the file name for the current model unit
		currentModel.appendToFileName(new String(ch, start, length).trim());
	}
	if (inBaseURI) {
		baseURI = new String(ch, start, length).trim();
	}
	super.characters(ch, start, length);
}

public void endElement(String uri, String localName, String qName) throws SAXException {
	if (ISMLConstants.GENICS_ELEMENT.equals(localName)) {
		inGenics = false;
	}
//	if (ISMLConstants.PHENICS_ELEMENT.equals(localName)) {
//		inPhenics = false;
//	}
	if (ISMLConstants.DATA_ELEMENT.equals(localName)) {
		inData = false;
		currentModel.stop();
		modelUnits.add(currentModel);
		currentModel = null;
	}
	if (ISMLConstants.LOCATOR_ELEMENT.equals(localName)) {
//		inLocator = false;
		if (currentModel != null) {
			modelUnits.add(currentModel);
			currentModel = null;
		}
	}
	if (ISMLConstants.BASE_URI_ELEMENT.equals(localName)) {
		inBaseURI = false;
	}
	if (ISMLConstants.ALIASES_ELEMENT.equals(localName)) {
		inAliases = false;
	}
	if (inData) {
    	writeToModelUnitFile("</"+qName+">");
	}
	super.endElement(uri, localName, qName);
}

public String getModelLocation() {
	if (baseURI == null) {
		return "";
	}
	String result = baseURI;
	if (result.startsWith("http://")) {
		result = result.substring("http://".length());
		int nextSlash = result.indexOf('/');
		if (nextSlash == -1) {
			result = "";
		} else {
			result = result.substring(nextSlash) + File.separatorChar;
		}
	}
	return result;
}

public List getModelUnits() {
	return modelUnits;
}

public boolean inModelUnit() {
	return buildingModelUnit;
}

public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
//	System.out.println("URI = "+uri+ " localName = "+localName+" qName = "+qName);
	if (ISMLConstants.GENICS_ELEMENT.equals(localName)) {
		inGenics = true;
	}
//	if (ISMLConstants.PHENICS_ELEMENT.equals(localName)) {
//		inPhenics = true;
//	}
	if (ISMLConstants.BASE_URI_ELEMENT.equals(localName)) {
		inBaseURI = true;
	}
	if (ISMLConstants.DOCUMENT_ELEMENT.equals(localName)) {
		currentModel = new ModelUnit(inGenics ? ModelUnit.GENIC : ModelUnit.PHENIC);
		currentModel.start();
	}
	if (ISMLConstants.LOCATOR_ELEMENT.equals(localName)) {
		// Need to retrieve remote document content
//		inLocator = true;
		String remoteDocUrlString = attributes.getValue(ISMLConstants.XLINK_URI, ISMLConstants.HREF_ATTRIBUTE);
		URL remoteDocUrl = null;
		try {
			remoteDocUrl = new URL(remoteDocUrlString);
		} catch (MalformedURLException e1) {
			// Bogus url
			currentModel = null;
			throw new ImportException(NLS.bind(SMLMessages.importErrorInvalidLocatorURL, remoteDocUrlString));
		}
		String localFileName = remoteDocUrl.getFile();
		currentModel.appendToFileName(localFileName);
		try {
			// TODO need to actually create a local file from the remote document
			SMLCommonUtil.retrieveRemoteDocument(remoteDocUrlString);
		} catch (RemoteRetrievalException e) {
			// Can't find document
			currentModel = null;
			
			// where should this be logged?
			throw new ImportException(NLS.bind(SMLMessages.importErrorMissingFileAtLocatorURL, remoteDocUrlString));
		}
	}
	if (ISMLConstants.ALIASES_ELEMENT.equals(localName)) {
		// This helps us know we need to get the fileName data out of the aliases
		inAliases = true;
	}
	if (inData) {
		buildingModelUnit = true;
        writeToModelUnitFile("<"+qName);
        if (attributes != null) {
            for (int i = 0; i < attributes.getLength(); i++) {
                String aName = attributes.getQName(i);
                writeToModelUnitFile(" ");
                writeToModelUnitFile(aName+"=\""+attributes.getValue(i)+"\"");
            }
        }
        writeToModelUnitFile(">");
	}
	if (ISMLConstants.DATA_ELEMENT.equals(localName)) {
		inData = true;
	}
	super.startElement(uri, localName, qName, attributes);
}

private void writeToModelUnitFile(String string) {
	currentModel.appendToContents(string);
}
}
