/**********************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.validation.internal.core;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import org.eclipse.cosmos.rm.validation.internal.common.AbstractValidationOutput;
import org.eclipse.cosmos.rm.validation.internal.common.FileOutput;
import org.eclipse.cosmos.rm.validation.internal.common.IValidationConstants;
import org.eclipse.cosmos.rm.validation.internal.common.MarkerOutput;
import org.eclipse.cosmos.rm.validation.internal.common.SMLMessages;
import org.eclipse.cosmos.rm.validation.internal.common.SystemOutput;
import org.eclipse.cosmos.rm.validation.internal.smlvalidators.AcyclicValidator;
import org.eclipse.cosmos.rm.validation.internal.smlvalidators.IdentityConstraintValidator;
import org.eclipse.cosmos.rm.validation.internal.smlvalidators.SchematronValidator;
import org.eclipse.cosmos.rm.validation.internal.smlvalidators.TargetValidator;


/**
 * This is the starting point for a validation process.  Clients are encouraged to re-use the same instance
 * with the {@link this#initialize(Map)} and/or {@link this#setAttribute(String, String)} methods 
 * to configure an instance based on a specific setting.  
 * 
 * @author Ali Mehregani
 */
public class MainValidator
{	
	private static final String OPTION_XML = "-xml";
	private static final String OPTION_SML = "-sml";
	private static final String OPTION_SCHEMATRON = "-schematron";
	private static final String OPTION_FILE = "-output";
	
	/**
	 * The attributes of this validation class
	 */
	private Map attributes;
	
	
	/**
	 * The constructor
	 */
	public MainValidator()
	{
		this.attributes = new Hashtable();
		setDefaultAttributes();
	}
	
	
	/**
	 * Sets default values for attributes
	 */
	protected void setDefaultAttributes()
	{
		/* Standalone environment */
		attributes.put(IValidator.ATTRIBUTE_ENV, IValidator.VALUE_ENV_STANDALONE);
		
		/* Default XML validator */
		attributes.put(IValidator.ATTRIBUTE_VALDIATION_XML, IXMLValidator.DEFAULT_VALIDATION_XML_LITERAL);
		
		/* Default SML validators */
		List smlValidators = new ArrayList();
		smlValidators.add(AcyclicValidator.class.getName());
		smlValidators.add(IdentityConstraintValidator.class.getName());
		smlValidators.add(TargetValidator.class.getName());
		smlValidators.add(SchematronValidator.class.getName());
		attributes.put(IValidator.ATTRIBUTE_VALDIATION_SML, smlValidators);
	}


	/**
	 * The initialize method is used to pass in multiple attributes
	 * at the same time
	 * 
	 * @param attributes The attributes.  The key and value pairs are expected 
	 * to be of type String
	 */
	public void initialize(Map attributes)
	{
		for (Iterator keys = attributes.keySet().iterator(); keys.hasNext();)
		{
			String currentKey = (String)keys.next();
			setAttribute((String)currentKey, (String)attributes.get(currentKey));
		}
	}


	/**
	 * Set an attribute of this type.  The key must be one of the ATTRIBUTE_* 
	 * constants defined by this class.
	 * 
	 * @param key The attribute key (see the ATTRIBUTE_* constants of this class)
	 * @param value The value of the attribute
	 */
	public void setAttribute(String key, String value)
	{
		attributes.put(key, value);
	}

	
	/**
	 * Start the validation process
	 */
	public boolean start()
	{
		/* Before we get started, we need to setup a common outstream where messages 
		 * will be logged to */
		AbstractValidationOutput validationOutput = createValidationOutput();
		attributes.put(IValidator.ATTRIBUTE_OUTPUT, validationOutput);
		Exception exception = null;
		
		try
		{
			//There are three phases to validate a SML document
			//Phase 1: create data structures that will be used in the sml validation phase.  Note: In this phase W3CSchema language validation will occur.		
			//DatastructureBuilder is always needed.
			IValidator datastructurBuilder = new DatastructureBuilderImpl();
	
			//Phase 2: sml validation phase
			IValidator[] smlValidations = ValidationFactory.createValidator(ISMLValidator.class, attributes.get(IValidator.ATTRIBUTE_VALDIATION_SML));
			
			//Phase 3: schematron validation phase
			//Note: schematron validation is now a builder
			IValidator[] schematronValidation = ValidationFactory.createValidator(ISchematronValidation.class, attributes.get(IValidator.ATTRIBUTE_VALDIATION_SCHEMATRON));
									
			IValidator[] completeSet = new IValidator[1 + smlValidations.length + schematronValidation.length];
			completeSet[0] = datastructurBuilder;
			System.arraycopy(smlValidations, 0, completeSet, 1, smlValidations.length);
			System.arraycopy(schematronValidation, 0, completeSet, 1 + smlValidations.length, schematronValidation.length);
			
			return startValidation(true, completeSet) && startValidation(false, completeSet);
		} 
		catch (InstantiationException e)
		{
			exception = e;
		} 
		catch (IllegalAccessException e)
		{
			exception = e;
		} 
		catch (ClassNotFoundException e)
		{
			exception = e;
		}
		finally
		{
			if (exception != null)
			{
				String message = exception.getMessage();
				validationOutput.reportError(SMLMessages.errorValidatorInstantiation + (message == null ? IValidationConstants.EMPTY_STRING : message));
			}
			validationOutput.close();
		}
		
		return false;
	}	
	
	
	private boolean startValidation(boolean initialize, IValidator[] validators)
	{				
		boolean status = true;
		for (int i = 0; i < validators.length; i++)
		{
			if (validators[i] != null)
			{
				if (initialize)
				{
					validators[i].initialize(attributes);
				}
				else
				{
					status = validators[i].validate();	
					if (!status)
						return status;
				}					
			}
		}		
		return status;
	}
	


	/**
	 * Create a validation output based on the environment that the validation 
	 * is being performed in.
	 * 
	 * @return An output stream
	 */
	protected AbstractValidationOutput createValidationOutput()
	{
		String environment = (String)attributes.get(IValidator.ATTRIBUTE_ENV);
		AbstractValidationOutput validationOutput = null;
		/* Eclipse */
		if (IValidator.VALUE_ENV_ECLIPSE.equals(environment))
		{
			validationOutput = new MarkerOutput();
		}
		/* Standalone */
		else if (IValidator.VALUE_ENV_STANDALONE.equals(environment))
		{
			validationOutput =  new FileOutput();
		}
		/* Unspecified - system output by default */
		else
		{
			validationOutput =  new SystemOutput();
		}
		validationOutput.initialize(attributes);
		return validationOutput;
	}


	/**
	 * Returns the attributes
	 * 
	 * @return attributes
	 */
	public Map getAttributes()
	{
		return attributes;
	}
	
	
	/**
	 * The main method - Used to set the appropriate attributes based on the
	 * program arguments.  Expected format:
	 * org.eclipse.cosmos.rm.validation.internal.core.MainValidator ([option][option argument(s)])* <path to XML document>,
	 * where available option and arguments are:
	 * <ul>
	 * 	<li> -xml <a fully qualified class name> </li>
	 * 	<li> -sml <fully qualified class names separated by a comma> </li>
	 * 	<li> -schematron <a fully qualified class name> </li>
	 *  <li> -ouptut <path to the output file> </li>
	 * </ul>
	 * 
	 * @param args The program arguments
	 */
	public static void main (String[] args)
	{
		if (args.length <= 0)
		{
			printUsage();
			return;
		}
		
		int lastInx = -1, currentInx = 0;
		MainValidator mainValidator = new MainValidator();
		
		try
		{
			while (lastInx != currentInx)
			{
				lastInx = currentInx;
				currentInx = processOption(mainValidator, args, currentInx);
			}
			
			if (currentInx != args.length - 1)
			{
				printUsage();
				return;
			}
			
			mainValidator.getAttributes().put(IValidator.ATTRIBUTE_INSTANCE, args[currentInx]);			
		}
		catch (IllegalArgumentException e)
		{
			printUsage();
		}		
		
		mainValidator.start();
	}


	private static int processOption(MainValidator mainValidator, String[] args, int inx) throws IllegalArgumentException
	{
		if (inx < 0 || inx >= args.length)
			return inx;
		
		String currentOption = args[inx];
		boolean xml = false;
		
		if ((xml = OPTION_XML.equals(currentOption)) || OPTION_SCHEMATRON.equals(currentOption))
		{	
			/* The next argument is expected to be the XML validator class */
			inx = addAttribute(mainValidator, args, ++inx, xml ? IValidator.ATTRIBUTE_VALDIATION_XML : IValidator.ATTRIBUTE_VALDIATION_SCHEMATRON);			
		}
		else if (OPTION_SML.equals(currentOption))
		{
			List smlValidator = new ArrayList();
			while (++inx < args.length - 1 && !args[inx].startsWith("-"))
			{
				StringTokenizer st = new StringTokenizer(args[inx]);
				while (st.hasMoreTokens())
				{
					smlValidator.add(st.nextToken());
				}
			}
			
			if (smlValidator.isEmpty())
				throw new IllegalArgumentException();
			
			mainValidator.getAttributes().put(IValidator.ATTRIBUTE_VALDIATION_SML, smlValidator);
		}
		else if (OPTION_FILE.equals(currentOption))
		{
			inx = addAttribute(mainValidator, args, ++inx, IValidator.ATTRIBUTE_FILE_OUTPUT);						
		}
		
		return inx;
	}


	private static int addAttribute(MainValidator mainValidator, String[] args, int inx, String attribute) throws IllegalArgumentException
	{
		if (inx >= args.length || args[inx].startsWith("-"))
			throw new IllegalArgumentException();
		
		mainValidator.getAttributes().put(attribute, args[inx]);
		return inx + 1;
	}


	private static void printUsage()
	{
		System.err.println(SMLMessages.errorIllegalArguments);
	}
}
