/**********************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.validation.internal.smlvalidators;

import java.io.StringReader;
import java.io.StringWriter;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.NamespaceContext;
import javax.xml.transform.ErrorListener;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;

import org.eclipse.cosmos.rm.validation.internal.artifacts.ElementTypeMap;
import org.eclipse.cosmos.rm.validation.internal.artifacts.SchematronNode;
import org.eclipse.cosmos.rm.validation.internal.artifacts.TypeNode;
import org.eclipse.cosmos.rm.validation.internal.common.ISMLConstants;
import org.eclipse.cosmos.rm.validation.internal.common.IValidationConstants;
import org.eclipse.cosmos.rm.validation.internal.common.IValidationOutput;
import org.eclipse.cosmos.rm.validation.internal.common.SMLCommonUtil;
import org.eclipse.cosmos.rm.validation.internal.core.AbstractSMLValidator;
import org.eclipse.cosmos.rm.validation.internal.databuilders.DataBuilderRegistry;
import org.eclipse.cosmos.rm.validation.internal.databuilders.DocumentDOMBuilder;
import org.eclipse.cosmos.rm.validation.internal.databuilders.ElementSchematronCacheBuilder;
import org.eclipse.cosmos.rm.validation.internal.databuilders.ElementTypeMapDataBuilder;
import org.eclipse.cosmos.rm.validation.internal.databuilders.IdentityDataBuilder;
import org.eclipse.cosmos.rm.validation.internal.databuilders.NamespaceContextBuilder;
import org.eclipse.cosmos.rm.validation.internal.databuilders.TypeInheritanceDataBuilderImpl;
import org.eclipse.cosmos.rm.validation.internal.util.FileHelper;
import org.eclipse.cosmos.rm.validation.internal.util.ParserHelper;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * This validator will apply schematrons to elements whose schema
 * has a schemtron. 
 * 
 * @author sleeloy
 */
public class SchematronValidator extends AbstractSMLValidator {
	//currently hard code the location of the style sheet to extract the schematron from the schema file
	protected final static String SKELETON15="schematron\\skeleton1-5.xsl";
	
	protected IValidationOutput validationLogger;
	protected ElementTypeMap elementTypeMapBuilderStructure;
	protected Map schemaCacheBuilderStructure;	
	protected Map inheritance;		
 
	public void initialize(Map validationAttribute)
	{
		super.initialize(validationAttribute);
		DataBuilderRegistry builderRegistry = DataBuilderRegistry.instance();
		
		builderRegistry.registerDataStructureBuilder(ElementSchematronCacheBuilder.ID, new ElementSchematronCacheBuilder());
		builderRegistry.registerDataStructureBuilder(ElementTypeMapDataBuilder.ID, new ElementTypeMapDataBuilder());
		builderRegistry.registerDataStructureBuilder(DocumentDOMBuilder.ID, new DocumentDOMBuilder());				
		builderRegistry.registerDataStructureBuilder(IdentityDataBuilder.ID, new IdentityDataBuilder());
		builderRegistry.registerDataStructureBuilder(TypeInheritanceDataBuilderImpl.ID, new TypeInheritanceDataBuilderImpl());
		builderRegistry.registerDataStructureBuilder(NamespaceContextBuilder.ID, new NamespaceContextBuilder());
		
						
	}

	public boolean validate()
	{			
		validationLogger = getValidationOutput();
				
		inheritance = (Map)SMLCommonUtil.retrieveDataStructure(TypeInheritanceDataBuilderImpl.ID);
		elementTypeMapBuilderStructure = (ElementTypeMap)SMLCommonUtil.retrieveDataStructure(ElementTypeMapDataBuilder.ID);
		Map domDocuments = (Map)SMLCommonUtil.retrieveDataStructure(DocumentDOMBuilder.ID);
		schemaCacheBuilderStructure = (Map)SMLCommonUtil.retrieveDataStructure(ElementSchematronCacheBuilder.ID);	
		List docWithNoAlias = (List)domDocuments.get(DocumentDOMBuilder.NO_ALIAS);

		//Walk through documents and determine which elements to apply the schematron 
		Iterator iter = domDocuments.keySet().iterator();
		while (iter.hasNext()){
			Object alias = iter.next();
			Object rootElement = domDocuments.get(alias);			
			if (rootElement instanceof Element)
			{
				if (!processNode((Element)rootElement, (Node)rootElement, alias))
				{
					return false;
				}
			}
		}
		
		//Walk through documents with no alias and determine which elements to apply the schematron 
		iter = docWithNoAlias.iterator();
		while (iter.hasNext()){
			Object rootElement = iter.next();
			if (rootElement instanceof Element)
			{
				if (!processNode((Element)rootElement, (Node)rootElement, null))
					return false;
			}
		}
		
		return true;
	}
	
	/**
	 * Processes a node to determine if a schematron should be applied.  
	 * @param currentElement node to process
	 * @param schemaCacheBuilderStructure  map containing schematron keyed on namespace uri
	 * @param rootElement root document element associated with the current element
	 * @param alias alias name of the root document.  This is needed if rule binding is defined in document
	 * @return true if node was processed successfully otherwise false is returned.
	 */
	protected boolean processNode(Node currentElement, Node rootElement, Object alias){
		
		if (currentElement.getNodeType() != Node.ELEMENT_NODE)
			return true;
		
		//check to see if this element has a schematron
		Map uriLookup = (Map)schemaCacheBuilderStructure.get(currentElement.getNamespaceURI());
		if (uriLookup == null) 
			return true;
		
		SchematronNode schematron = (SchematronNode)uriLookup.get(currentElement.getNodeName());
		//check alias
		if ((alias != null) && (schematron == null)){
			schematron = (SchematronNode)uriLookup.get(alias);
		}
		if (schematron == null){
			//check to see if schematron is associated with the type
			Object getTargetType = elementTypeMapBuilderStructure.getType(currentElement.getNamespaceURI(), currentElement.getLocalName());			
			
			if (getTargetType != null){
				schematron = (SchematronNode)uriLookup.get(getTargetType+ISMLConstants.TYPEDELIM);
			
				//check inheritance tree
				if (schematron == null){
					Map inheritanceMap = (Map)inheritance.get(currentElement.getNamespaceURI());
					if (inheritanceMap != null){
						TypeNode parent = (TypeNode)inheritanceMap.get(getTargetType);
						while (parent != null){
							schematron = (SchematronNode)uriLookup.get(ParserHelper.removeNameSpace(parent.getType())+ISMLConstants.TYPEDELIM);
							if (schematron != null){
								break;
							}
							parent = (TypeNode)inheritanceMap.get(parent.getType());
							
						}
					}
				}										
			}
		}
		
		//keep track of xpath in builder
		if (schematron != null) {
			if (!(validateSchematron(schematron, currentElement, rootElement)))
				return false;
		}
		
		//check children elements to see if a schematron should be applied
		NodeList nodeLists = currentElement.getChildNodes();
		for (int x = 0; x <nodeLists.getLength(); x++){
			Node node = nodeLists.item(x);
			if (node.getNodeType() == Node.ELEMENT_NODE)
			{					
				if (!processNode(node, rootElement, null))
					return false;
			}
		}
		
		
		return true;				
	}
	
	protected boolean validateSchematron(SchematronNode schematronNode, Node currentElement, Node rootElement){
		try {						
	    	StringBuffer buffer = schematronNode.getSchematron();
			//valid if there's no schema to validate against
			if (buffer == null) return true;

			String schematronFragment = buffer.toString().replaceAll("\\"+ISMLConstants.CONTEXTPARAM, "/*[1]");
			
			StringReader sr = new StringReader(schematronFragment);

			//extract xsl from schematron
			DOMResult schematron = new DOMResult();				
			transform(new StreamSource(sr), new StreamSource(FileHelper.getResourceAsStream(SKELETON15)), schematron);
			
			//evaluate ruleContext
			String ruleContext = schematronNode.getRuleContext();

			final Map prefixMap = schematronNode.getPrefixMap();
			if (ruleContext.length() > 0){
				NamespaceContext namespaceContext = new NamespaceContext(){
	
					public String getNamespaceURI(String prefix)
					{
						String uri = (String)prefixMap.get(prefix);
						return uri == null ? IValidationConstants.EMPTY_STRING : uri;
					}
	
					public String getPrefix(String arg0)
					{
						return null;
					}
	
					public Iterator getPrefixes(String arg0)
					{
						return null;
					}};
					
				SMLCommonUtil.xpath.setNamespaceContext(namespaceContext);
				
				XPathExpression xpathExp = SMLCommonUtil.xpath.compile(ruleContext);
				
				NodeList nodeList = (NodeList)xpathExp.evaluate(currentElement, XPathConstants.NODESET);
				if (nodeList != null){
					//apply schematron on the nodes returned by evaluating the rule context 
					for (int x = 0; x < nodeList.getLength(); x++)
					{
						if (!applySchematron(nodeList.item(x), schematron.getNode()))
						{
							return false;
						}
					}
				}
			}
			else
			{
				//apply schematron on the current element
				if (!applySchematron(currentElement, schematron.getNode()))
					return false;
			}
			return true;
		} catch (Exception e) {
			e.printStackTrace();
			validationLogger.reportError(e.getLocalizedMessage());
		}
		return false;
	}
	
	private boolean applySchematron(Node currentElement, Node schematron){
		StringWriter sw = new StringWriter();
		StreamResult output = new StreamResult(sw);
		DOMSource schematronSource =  new DOMSource(schematron);
				
		//apply schematron to main document
		transform(createStreamSource(currentElement), schematronSource, output);
		//check to see if this is an empty string
		String result = sw.getBuffer().toString();
		
		if (result.trim().length() > 0 )
		{
			validationLogger.reportError(result);
			return false;
		}
		return true;
	}
	
	
	private StreamSource createStreamSource(Node node){
		try {
			Transformer transformer = TransformerFactory.newInstance().newTransformer();

			//	initialize StreamResult with File object to save to file
			StreamResult result = new StreamResult(new StringWriter());
			DOMSource source = new DOMSource(node);
			transformer.transform(source, result);

			return new StreamSource(new StringReader(result.getWriter().toString()));
		} catch (TransformerConfigurationException e) {
			validationLogger.reportError(e.getLocalizedMessage());
		} catch (IllegalArgumentException e) {
			validationLogger.reportError(e.getLocalizedMessage());
		} catch (TransformerFactoryConfigurationError e) {
			validationLogger.reportError(e.getLocalizedMessage());
		} catch (TransformerException e)
		{
			validationLogger.reportError(e.getLocalizedMessage());
		}		
		return null;
	}
	/**
	 * Helper method that will apply a xslt transform to a given xml source and write the results
	 * to a Result object
	 * @param source xml source 
	 * @param xslt xslt transform that will be applied to the source
	 * @param result stores the resulting transform
	 */
	protected void transform(Source source, Source xslt, Result result) {
		Transformer transformer = null;

		// Instantiate a TransformerFactory
		TransformerFactory tFactory = TransformerFactory.newInstance();

		// Use the TransformerFactory to process the stylesheet source and
		// generate a Transformer.
		try {
			transformer = tFactory.newTransformer(xslt);
			transformer.setErrorListener(new ErrorListener(){

				public void error(TransformerException e) throws TransformerException {
					validationLogger.reportError(e.getLocalizedMessage());
				}

				public void fatalError(TransformerException e) throws TransformerException {
					validationLogger.reportError(e.getLocalizedMessage());
				}

				public void warning(TransformerException e) throws TransformerException {
					validationLogger.reportError(e.getLocalizedMessage());
				}
			}
			);
			
			// Use the Transformer to transform an XML Source and send the
			// output to a Result object.
			transformer.transform(source, result);

		} catch (TransformerConfigurationException e) {
			validationLogger.reportError(e.getLocalizedMessage());
		} catch (TransformerException e) {
			validationLogger.reportError(e.getLocalizedMessage());
		}
	}
	
	
}
